/**
 * \file tab.h
 * Tab contains all widgets within a single tab.
 *
 * \b Project: Qoobar
 * \author Alex Novichkov
 * \date 8 Dec 2009
 *
 * Copyright (C) 2009  Alex Novichkov
 *
 * This file is part of Qoobar.
 *
 * Qoobar is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Qoobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef TAB_H
#define TAB_H

#include <QWidget>
#include <QIcon>
#include <QTreeWidget>
#include <QUndoStack>
#include <QItemDelegate>
#include "tagger.h"

class QTableWidgetItem;
class QTreeWidgetItem;
class QLabel;
class ClickableLabel;
class QPushButton;
class QToolButton;
class MainWindow;
class Settings;
class QUndoCommand;
class QDropEvent;
class TreeWidget;
class TableWidget;
class QSplitter;
class ImageBox;
class QSignalMapper;
//class DelayedExecutionTimer;
class ChangeFilesHelper;
class BufferItem;
class IQoobarPlugin;
class Model;

struct BufferItem {
    BufferItem(int r,const QString &h, const QStringList &v)
        : row(r), header(h), values(v) {}
    int row;
    QString header;
    QStringList values;
};

class Tab : public QWidget
{
Q_OBJECT
public:
    explicit Tab(MainWindow *parent = 0);
    //virtual ~Tab() {}
    void addFiles(const QStringList &, bool clearBefore=false);
    void addFiles(const QList<Tag> &);

    void setTags(const QList<int> &, const QList<Tag> &,const QList<int> &);
    bool maybeSave(const QString &);
    bool allFilesSaved() const;
    void setTabNumber(int n) {tabnumber=n;}
    int tabNumber() {return tabnumber;}
    bool filesSelected();
    bool modelNotEmpty();
    void setStatus();
    void retranslateUi();
    void doPlugin(IQoobarPlugin *);

    //void setTableUpdateMode(bool useDelayedUpdate);
    void updateDirView(bool visible);
    QMap<int, QString> allTabsNames();
    bool allSelectedFilesAreMP3() const;

    QUndoStack *undoStack();
    Model *model;

    bool wasRenamed;
signals:
    void selectionChanged(bool);
    void filesChanged(bool);
    void bufferChanged(bool);
    void tagsSelectionChanged(bool tagsSelected);
    void filesCountChanged(bool);
    void totalLengthChanged(int);
    void moveFilesToTab(int,const QList<Tag>&);
    void updateStatusBar(const Tag &);
public slots:
    void copy();
    void paste();
    void cut();
    void newTag();
    void rereadTags();
    void renameFiles();
    void delFiles();
    void delAllFiles();
    void fill();
    void removeAllTags();
    void saveTags();
    void play();
    void handleCutCopy();
    void replaygain();
private slots:
    void filesSelectionChanged();
    void cellChanged(QTableWidgetItem *);
    void moveUp();
    void moveDown();
    void editCell();
    void pasteFromClipboard();
    void copyToClipboard();
    void remove();
    void onItemClicked(QTreeWidgetItem *,int);
    void updateStatusBar(QTreeWidgetItem *);
    void toggleImageBoxVisibility();
    void tagsChanged(int,const QStringList &);
    void saveSplitterState();
    void handleOperation(const QString &);
    void changeImage(const CoverImage &,const QString &);
    void collectTags();
    void updateTagsTableRow(int row, const QStringList &list);
    void moveToTab(int);
    void showMP3TagsDialog();
    void showMessage(int type,const QString &text);
private:
    void updateFilesTable(const QList<int> &, const QList<int> &);
    void updateTagsTable(const QList<int> & rows);

    void updateImageBox();
    void updateTags(QList<Tag> &oldTags, QList<Tag> &newTags, const QString &motivation);
    void rereadTags(bool all,bool force=true);
    void delFiles(bool silently);
    QStringList getTags(int);
    void moveItems(bool up);
    void rename(const QStringList &);
    void changeRow(ChangeFilesHelper *helper, int row, const QString &s, const QStringList &newValues);
    void pushCommand(ChangeFilesHelper *helper, const QString &motivation);

    static QList<BufferItem> internalBuffer;

    TableWidget *table;
    TreeWidget *tree;
    MainWindow *win;

    QPushButton *showImageBoxButton;

    QAction *moveUpAct;
    QAction *moveDownAct;

    QAction *editAct;
    QAction *multilineEditAct;
    QAction *removeAct;
    QAction *operationsAct;

    QAction *collectAct;


    ImageBox *imageBox;
    //DelayedExecutionTimer* delay;

    QSignalMapper *operationsMapper;
    QUndoStack *undoStack_;
    int tabnumber;
};

#endif // TAB_H
